require "qiita"

module Ruboty
  module Handlers
    class QiitaPolice < Base
      BRAIN_NAMESPACE = "qiita-police"

      REASONS_TABLE = {
        "bug" => "bugreport",
        "notprogram" => "notprogram",
        "request" => "featurerequest",
        "test" => "test",
        "tos" => "tosviolation",
      }

      on(
        %r<ban http://qiita\.com/.+/items/(?<item_id>[0-9a-f]+) +for (?<reason>#{REASONS_TABLE.keys.join("|")})\z>,
        description: "Ban item",
        name: :ban,
      )

      on(
        /save my qiita access token (?<token>.+)/,
        description: "Save sender's Qiita access token",
        name: :save,
      )

      def ban(message)
        case
        when (client = client_for(message.from_name)).nil?
          message.reply("I don't know your Qiita access token")
        when !REASONS_TABLE.include?(message[:reason])
          false
        else
          response = client.put(
            "/api/v2/items/#{message[:item_id]}/ban",
            reason: REASONS_TABLE[message[:reason]],
          )
          if response.status == 204
            message.reply("Banned")
          else
            message.reply(response.body["message"])
          end
        end
      end

      def save(message)
        access_tokens_table[message.from_name] = message[:token]
        message.reply("Saved #{message.from_name}'s qiita access token")
      end

      private

      def access_tokens_table
        robot.brain.data[BRAIN_NAMESPACE] ||= {}
      end

      def client_for(username)
        if access_token = access_tokens_table[username]
          Qiita::Client.new(access_token: access_token)
        end
      end
    end
  end
end
